@@ -16,6 +16,7 @@
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA               *
 **************************************************************************/
 #include <cstdio>
+#include <cstring>
 
 #include <sys/file.h>
 #include <unistd.h>
@@ -56,7 +57,7 @@ struct FileImpl
 FileImpl::FileImpl(const string& path_, const char* mode, string& openError)
 	: handle(NULL), buf(NULL), bufLength(0), path(path_), isPipe(false)
 {
-	if (strcmp(mode, "pr") == 0)
+	if (std::strcmp(mode, "pr") == 0)
 	{
 		// need to open read pipe
 		isPipe = true;
@@ -276,7 +277,7 @@ void File::lock(int flags)
 {
 	__impl->assertFileOpened();
 	int fd = __guarded_fileno(__impl->handle, __impl->path);
-	// TODO/2.1: consider using fcntl
+	// TODO/API break/: provide only lock(void) and unlock(void) methods, consider using fcntl
 	if (flock(fd, flags) == -1)
 	{
 		const char* actionName = (flags & LOCK_UN) ? "release" : "obtain";
