@@ -15,16 +15,16 @@
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA               *
 **************************************************************************/
-// TODO/2.1: define options for const numbers used in resolver
 
 #include <cmath>
 #include <queue>
 #include <algorithm>
 
+#include <common/regex.hpp>
+
 #include <cupt/config.hpp>
 #include <cupt/cache.hpp>
 #include <cupt/cache/binarypackage.hpp>
-#include <cupt/regex.hpp>
 
 #include <internal/nativeresolver/impl.hpp>
 #include <internal/graph.hpp>
@@ -213,7 +213,7 @@ bool NativeResolverImpl::__is_candidate_for_auto_removal(const dg::Element* elem
 		const std::function< bool (const string&) > isNeverAutoRemove,
 		bool canAutoremove)
 {
-	auto versionVertex = dynamic_cast< const dg::VersionVertex* >(*elementPtr);
+	auto versionVertex = dynamic_cast< const dg::VersionVertex* >(elementPtr);
 	if (!versionVertex)
 	{
 		return false;
@@ -300,15 +300,15 @@ void NativeResolverImpl::__clean_automatically_installed(Solution& solution)
 			{
 				continue; // main vertex
 			}
-			const list< const dg::Element* >& successorElementPtrs =
+			const GraphCessorListType& successorElementPtrs =
 					__solution_storage->getSuccessorElements(*elementPtrIt);
 			FORIT(successorElementPtrIt, successorElementPtrs)
 			{
-				if ((**successorElementPtrIt)->isAnti())
+				if ((*successorElementPtrIt)->isAnti())
 				{
 					continue;
 				}
-				const list< const dg::Element* >& successorSuccessorElementPtrs =
+				const GraphCessorListType& successorSuccessorElementPtrs =
 						__solution_storage->getSuccessorElements(*successorElementPtrIt);
 				FORIT(successorSuccessorElementPtrIt, successorSuccessorElementPtrs)
 				{
@@ -343,7 +343,7 @@ void NativeResolverImpl::__clean_automatically_installed(Solution& solution)
 
 				if (debugging)
 				{
-					debug("auto-removed '%s'", (**elementPtrIt)->toString().c_str());
+					__mydebug_wrapper(solution, sf("auto-removed '%s'", (*elementPtrIt)->toString().c_str()));
 				}
 				__solution_storage->setPackageEntry(solution, emptyElementPtr,
 						std::move(packageEntry), *elementPtrIt);
@@ -411,8 +411,8 @@ void NativeResolverImpl::__pre_apply_action(const Solution& originalSolution,
 	{
 		auto message = sf("-> (%u,Δ:[%s]) trying: '%s' -> '%s'",
 				solution.id, __score_manager.getScoreChangeString(profit).c_str(),
-				oldElementPtr ? (*oldElementPtr)->toString().c_str() : "",
-				(*newElementPtr)->toString().c_str());
+				oldElementPtr ? oldElementPtr->toString().c_str() : "",
+				newElementPtr->toString().c_str());
 		__mydebug_wrapper(originalSolution, message);
 	}
 
@@ -431,7 +431,7 @@ void NativeResolverImpl::__calculate_profits(vector< unique_ptr< Action > >& act
 		{
 			return emptyVersion;
 		}
-		auto versionVertex = dynamic_cast< const dg::VersionVertex* >(*elementPtr);
+		auto versionVertex = dynamic_cast< const dg::VersionVertex* >(elementPtr);
 		if (!versionVertex)
 		{
 			return emptyVersion;
@@ -444,7 +444,7 @@ void NativeResolverImpl::__calculate_profits(vector< unique_ptr< Action > >& act
 	{
 		Action& action = **actionIt;
 
-		switch ((*action.newElementPtr)->getUnsatisfiedType())
+		switch (action.newElementPtr->getUnsatisfiedType())
 		{
 			case dg::Unsatisfied::None:
 				action.profit = __score_manager.getVersionScoreChange(
@@ -538,13 +538,15 @@ bool NativeResolverImpl::__makes_sense_to_modify_package(const Solution& solutio
 {
 	/* we check only successors with the same or bigger priority than
 	   currently broken one */
-	auto brokenElementTypePriority = (*brokenElementPtr)->getTypePriority();
+	auto brokenElementTypePriority = brokenElementPtr->getTypePriority();
 
-	const list< const dg::Element* >& successorElementPtrs =
+	__solution_storage->unfoldElement(candidateElementPtr);
+
+	const GraphCessorListType& successorElementPtrs =
 			__solution_storage->getSuccessorElements(candidateElementPtr);
 	FORIT(successorElementPtrIt, successorElementPtrs)
 	{
-		if ((**successorElementPtrIt)->getTypePriority() < brokenElementTypePriority)
+		if ((*successorElementPtrIt)->getTypePriority() < brokenElementTypePriority)
 		{
 			continue;
 		}
@@ -554,25 +556,25 @@ bool NativeResolverImpl::__makes_sense_to_modify_package(const Solution& solutio
 			{
 				__mydebug_wrapper(solution, sf(
 						"not considering %s: it has the same problem",
-						(*candidateElementPtr)->toString().c_str()));
+						candidateElementPtr->toString().c_str()));
 			}
 			return false;
 		}
 	}
 
 	// let's try even harder to find if this candidate is really appropriate for us
-	const list< const dg::Element* >& brokenElementSuccessorElementPtrs =
+	const GraphCessorListType& brokenElementSuccessorElementPtrs =
 			__solution_storage->getSuccessorElements(brokenElementPtr);
 	FORIT(successorElementPtrIt, successorElementPtrs)
 	{
-		if ((**successorElementPtrIt)->getTypePriority() < brokenElementTypePriority)
+		if ((*successorElementPtrIt)->getTypePriority() < brokenElementTypePriority)
 		{
 			continue;
 		}
 		/* if any of such successors gives us equal or less "space" in
 		   terms of satisfying elements, the version won't be accepted as a
 		   resolution */
-		const list< const dg::Element* >& successorElementSuccessorElementPtrs =
+		const GraphCessorListType& successorElementSuccessorElementPtrs =
 				__solution_storage->getSuccessorElements(*successorElementPtrIt);
 
 		bool isMoreWide = false;
@@ -596,7 +598,7 @@ bool NativeResolverImpl::__makes_sense_to_modify_package(const Solution& solutio
 			{
 				__mydebug_wrapper(solution, sf(
 						"not considering %s: it contains equal or less wide relation expression '%s'",
-						(*candidateElementPtr)->toString().c_str(), (**successorElementPtrIt)->toString().c_str()));
+						candidateElementPtr->toString().c_str(), (*successorElementPtrIt)->toString().c_str()));
 			}
 			return false;
 		}
@@ -639,7 +641,7 @@ void NativeResolverImpl::__add_actions_to_modify_package_entry(
 void NativeResolverImpl::__add_actions_to_fix_dependency(vector< unique_ptr< Action > >& actions,
 		const Solution& solution, const dg::Element* brokenElementPtr)
 {
-	const list< const dg::Element* >& successorElementPtrs =
+	const GraphCessorListType& successorElementPtrs =
 			__solution_storage->getSuccessorElements(brokenElementPtr);
 	// install one of versions package needs
 	FORIT(successorElementPtrIt, successorElementPtrs)
@@ -672,7 +674,7 @@ Resolver::UserAnswer::Type NativeResolverImpl::__propose_solution(
 	auto elementPtrs = solution.getElements();
 	FORIT(elementPtrIt, elementPtrs)
 	{
-		auto vertex = dynamic_cast< const dg::VersionVertex* >(**elementPtrIt);
+		auto vertex = dynamic_cast< const dg::VersionVertex* >(*elementPtrIt);
 		if (vertex)
 		{
 			const string& packageName = vertex->getPackageName();
@@ -708,18 +710,18 @@ Resolver::UserAnswer::Type NativeResolverImpl::__propose_solution(
 		else
 		{
 			// non-version vertex - unsatisfied one
-			const list< const dg::Element* >& predecessors =
+			const GraphCessorListType& predecessors =
 					__solution_storage->getPredecessorElements(*elementPtrIt);
 			FORIT(predecessorIt, predecessors)
 			{
-				const list< const dg::Element* >& affectedVersionElements =
+				const GraphCessorListType& affectedVersionElements =
 						__solution_storage->getPredecessorElements(*predecessorIt);
 				FORIT(affectedVersionElementIt, affectedVersionElements)
 				{
 					if (solution.getPackageEntry(*affectedVersionElementIt))
 					{
 						offer.unresolvedProblems.push_back(
-								(**predecessorIt)->getReason(***affectedVersionElementIt));
+								(*predecessorIt)->getReason(**affectedVersionElementIt));
 					}
 				}
 			}
@@ -761,7 +763,7 @@ void NativeResolverImpl::__generate_possible_actions(vector< unique_ptr< Action
 void NativeResolverImpl::__validate_element(
 		Solution& solution, const dg::Element* elementPtr, size_t priority)
 {
-	const list< const dg::Element* >& successorElementPtrs =
+	const GraphCessorListType& successorElementPtrs =
 			__solution_storage->getSuccessorElements(elementPtr);
 	forward_list< PackageEntry::BrokenSuccessor > brokenSuccessors;
 	FORIT(successorElementPtrIt, successorElementPtrs)
@@ -795,15 +797,15 @@ void NativeResolverImpl::__final_verify_solution(const Solution& solution)
 	auto elementPtrs = solution.getElements();
 	FORIT(elementPtrIt, elementPtrs)
 	{
-		const list< const dg::Element* >& successorElementPtrs =
+		const GraphCessorListType& successorElementPtrs =
 				__solution_storage->getSuccessorElements(*elementPtrIt);
 		FORIT(successorElementPtrIt, successorElementPtrs)
 		{
 			if (!__solution_storage->verifyElement(solution, *successorElementPtrIt))
 			{
 				fatal("internal error: final solution check failed: solution '%u', version '%s', problem '%s'",
-						solution.id, (**elementPtrIt)->toString().c_str(),
-						(**successorElementPtrIt)->toString().c_str());
+						solution.id, (*elementPtrIt)->toString().c_str(),
+						(*successorElementPtrIt)->toString().c_str());
 			}
 		}
 	}
@@ -817,13 +819,13 @@ void NativeResolverImpl::__validate_changed_package(Solution& solution,
 
 	if (oldElementPtr)
 	{ // invalidate those which depend on the old element
-		const list< const dg::Element* >& predecessors =
+		const GraphCessorListType& predecessors =
 				__solution_storage->getPredecessorElements(oldElementPtr);
 		FORIT(predecessorElementPtrIt, predecessors)
 		{
 			if (!__solution_storage->verifyElement(solution, *predecessorElementPtrIt))
 			{
-				const list< const dg::Element* >& dependentVersionElementPtrs =
+				const GraphCessorListType& dependentVersionElementPtrs =
 						__solution_storage->getPredecessorElements(*predecessorElementPtrIt);
 				FORIT(versionElementPtrIt, dependentVersionElementPtrs)
 				{
@@ -845,11 +847,11 @@ void NativeResolverImpl::__validate_changed_package(Solution& solution,
 		}
 	}
 	{ // validate those which depend on the new element
-		const list< const dg::Element* >& predecessors =
+		const GraphCessorListType& predecessors =
 				__solution_storage->getPredecessorElements(newElementPtr);
 		FORIT(predecessorElementPtrIt, predecessors)
 		{
-			const list< const dg::Element* >& dependentVersionElementPtrs =
+			const GraphCessorListType& dependentVersionElementPtrs =
 					__solution_storage->getPredecessorElements(*predecessorElementPtrIt);
 			FORIT(versionElementPtrIt, dependentVersionElementPtrs)
 			{
@@ -896,8 +898,8 @@ pair< const dg::Element*, PackageEntry::BrokenSuccessor > __get_broken_pair(
 	return *std::max_element(brokenPairs.begin(), brokenPairs.end(),
 			[failValue](const BrokenPairType& left, const BrokenPairType& right)
 			{
-				auto leftTypePriority = (*left.second.elementPtr)->getTypePriority();
-				auto rightTypePriority = (*right.second.elementPtr)->getTypePriority();
+				auto leftTypePriority = left.second.elementPtr->getTypePriority();
+				auto rightTypePriority = right.second.elementPtr->getTypePriority();
 				if (leftTypePriority < rightTypePriority)
 				{
 					return true;
@@ -927,8 +929,8 @@ pair< const dg::Element*, PackageEntry::BrokenSuccessor > __get_broken_pair(
 					return false;
 				}
 
-				return static_cast< const dg::VersionVertex* >(*left.first)->getPackageName() >
-						static_cast< const dg::VersionVertex* >(*right.first)->getPackageName();
+				return static_cast< const dg::VersionVertex* >(left.first)->getPackageName() >
+						static_cast< const dg::VersionVertex* >(right.first)->getPackageName();
 			});
 }
 
@@ -1008,9 +1010,9 @@ bool NativeResolverImpl::resolve(Resolver::CallbackType callback)
 			if (debugging)
 			{
 				auto message = sf("problem (%zu:%zu): %s: %s",
-						(*brokenSuccessor.elementPtr)->getTypePriority(), brokenSuccessor.priority,
-						(*versionElementPtr)->toString().c_str(),
-						(*brokenSuccessor.elementPtr)->toString().c_str());
+						brokenSuccessor.elementPtr->getTypePriority(), brokenSuccessor.priority,
+						versionElementPtr->toString().c_str(),
+						brokenSuccessor.elementPtr->toString().c_str());
 				__mydebug_wrapper(*currentSolution, message);
 			}
 			__generate_possible_actions(&possibleActions, *currentSolution,
